@@ -2,10 +2,11 @@
 Provides MathJax support for rendering Markdown with LaTeX to html.
 """
 
-from mistletoe.html_renderer import HTMLRenderer
+from mistletoe.html_renderer import HtmlRenderer
 from mistletoe.latex_renderer import LaTeXRenderer
 
-class MathJaxRenderer(HTMLRenderer, LaTeXRenderer):
+
+class MathJaxRenderer(HtmlRenderer, LaTeXRenderer):
     def __init__(self, **kwargs):
         """
         Args:
@@ -18,11 +19,13 @@ class MathJaxRenderer(HTMLRenderer, LaTeXRenderer):
 
     def render_math(self, token):
         """
-        Ensure Math tokens are all enclosed in two dollar signs.
+        Convert single dollar sign enclosed math expressions to the ``\\(...\\)`` syntax, to support
+        the default MathJax settings which ignore single dollar signs as described at
+        https://docs.mathjax.org/en/latest/basic/mathematics.html#tex-and-latex-input.
         """
         if token.content.startswith('$$'):
             return self.render_raw_text(token)
-        return '${}$'.format(self.render_raw_text(token))
+        return '\\({}\\)'.format(self.render_raw_text(token).strip('$'))
 
     def render_document(self, token):
         """
