@@ -12,19 +12,20 @@ def traverse(source, klass=None, depth=None, include_source=False):
         klass: filter children by a certain token class
         depth (int): The depth to recurse into the tree
         include_source (bool): whether to first yield the source element
+                               (provided it passes any given ``klass`` filter)
 
     Yields:
         A container for an element, its parent and depth
     """
     current_depth = 0
-    if include_source:
+    if include_source and (klass is None or isinstance(source, klass)):
         yield TraverseResult(source, None, current_depth)
     next_children = [(source, c) for c in getattr(source, 'children', [])]
-    while next_children and (depth is None or current_depth > depth):
+    while next_children and (depth is None or current_depth < depth):
         current_depth += 1
         new_children = []
         for parent, child in next_children:
-            if klass is None or issubclass(child, klass):
+            if klass is None or isinstance(child, klass):
                 yield TraverseResult(child, parent, current_depth)
             new_children.extend(
                 [(child, c) for c in getattr(child, 'children', [])]
